/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      generate.c,v
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    1.14
   Date:        1994/01/28 15:33:47


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   generate.c,v
 * Revision 1.14  1994/01/28  15:33:47  ware
 * Changes to help support SGML.  Installation Improvements.
 *
 * Revision 1.13  1993/04/19  17:11:55  ware
 * Merged John Boyds changes.
 *
 * Revision 1.12  92/03/16  15:01:25  ware
 * Make titles consistent.
 * 
 * Revision 1.11  1992/03/10  19:46:42  cso
 * Removed the plus operator to aid in symbol recognition
 *
 * Revision 1.10  1992/02/25  02:55:06  cso
 * Fixed a string compare of wrong types
 *
 * Revision 1.9  1992/02/20  04:55:54  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.8  1992/02/06  04:49:25  cso
 * Modified the End tag code per ultrix user comment
 *
 * Revision 1.7  1992/01/28  18:41:04  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "chtab.h"
#include "contab.h"
#include "output.h"
#include "retag.h"
#include "status.h"
#include "tables.h"



/*
   FUNCTION: 		Output

   Synopsis: To print static portions of the lex and yacc files.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
Output(fptr, str)

FILE
   *fptr;
String
   str[];

{
   int
      i;

   for (i = 0; str[i] != 0; i++)
   {
      fprintf(fptr, "%s\n", str[i]);
   }
   return;
}



/*
   FUNCTION: 		PrintStartConds

   Synopsis: To print the start condition in the generated lex code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintStartConds(fptr)

FILE
   *fptr;

{
   int i;
   
   if (Ctxtndx != 0)
   {
      fprintf(fptr, "%%START ");
      for (i = 0; i < Ctxtndx; i++)
      {
	 fprintf(fptr, "%s ", Ctxttab[i]);
      }
      fprintf(fptr, "\n");
   }
}



/*
   FUNCTION: 		PrintCdataRules

   Synopsis: To print the character data rules in the generated lex code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintCdataRules(fptr, chndx)

FILE
   *fptr;
int
   chndx;

{
   char
      abuff[15], /* buffer to build alphabetic cdata string in. */
      nbuff[10], /* buffer to build numeric cdata string in.     */
      sbuff[50]; /* buffer to build special cdata string in.     */

   MakeCdata(abuff, nbuff, sbuff, chndx);
   if (chndx == 0)
   {
      fprintf(fptr, "[%s]+\t{\n", abuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "[%s]+\t{\n", nbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "[%s]\t{\n", sbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
   }
   else
   {
      fprintf(fptr, "<%s>[%s]+\t{\n", Ctxttab[chndx - 1], abuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "<%s>[%s]+\t{\n", Ctxttab[chndx - 1], nbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "<%s>[%s]\t{\n", Ctxttab[chndx - 1], sbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
   }
   return;
}



/*
   FUNCTION: 		PrintSCLexRules

   Synopsis: To print the reg exp rules with start conditions
   in the generated lex code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintSCLexRules(fptr)

FILE
   *fptr;

{
   int
      i,
      j;

   /*
    * Every recognized token is being made into a string to be passed
    * over to the yacc side of things because it is only then that we
    * know whether the recognized token is ambiguous in that it could
    * either be in the text production or as part of a tag production.
    * On the yacc side these strings are currently NOT being freed
    * because of memory allocation efficiency, some day they may have
    * to be.
    */
   for (i = 0; i < Ctxtndx; i++)
   {
      for (j = 0; j < Lexndx; j++)
      {
	 if (strcmp(Lextab[j].context, Ctxttab[i]) == 0)
	 {
	    /* do not generate code for ICAEOF */
	    if (strcmp(Lextab[j].regexp, ICAEOF) != 0)
	    {
	       switch(Lextab[j].ccode)
	       {
	       default:
		  break;

	       case Stop:
		  fprintf(fptr,
			  "<%s>%s\t{\n", Lextab[j].context, Lextab[j].regexp);
		  fprintf(fptr, "\t BEGIN pop(%s);\n", Lextab[j].context);
		  fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
		  fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
		  fprintf(fptr, "\t return(TOKEN%d);\n", j);
		  fprintf(fptr, "\t}\n\n");
		  UpdateChartab(Lextab[j].regexp, i + 1);
		  break;

	       case In:
		  fprintf(fptr, "<%s>%s\t{\n",
			  Lextab[j].context, Lextab[j].regexp);
		  fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
		  fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
		  fprintf(fptr, "\t return(TOKEN%d);\n", j);
		  fprintf(fptr, "\t}\n\n");
		  UpdateChartab(Lextab[j].regexp, i + 1);
		  break;

	       case Nest:
		  fprintf(fptr, "<%s>%s\t{\n",
			  Lextab[j].context, Lextab[j].regexp);
		  /* have to access the new field in lex, whatever its */
		  /* name is. */
		  /* What will that new field be called?
		     fprintf(fptr, "\t BEGIN %s;\n", Lextab[i].context2);
		     fprintf(fptr, "\t push(%s);\n", Lextab[i].context2);
		     */
		  fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
		  fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
		  fprintf(fptr, "\t return(TOKEN%d);\n", j);
		  fprintf(fptr, "\t}\n\n");
		  UpdateChartab(Lextab[j].regexp, i + 1);
		  break;
	       }
	    }
	 }
      }
      PrintCdataRules(fptr, i + 1);
   }
   return;
}



/*
   FUNCTION: 		PrintLexRules

   Synopsis: To print the reg exp rules in the generated lex code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintLexRules(fptr)

FILE
   *fptr;

{
   int
      i;

   /*
    * Every recognized token is being made into a string to be passed
    * over to the yacc side of things because it is only then that we
    * know whether the recognized token is ambiguous in that it could
    * either be in the text production or as part of a tag production.
    * On the yacc side these strings are currently NOT being freed
    * because of memory allocation efficiency, some day they may have
    * to be.
    */
   for (i = Lexndx - 1; i >= 0; i--)
   {

      /* do not generate code for ICAEOF */
      if (strcmp(Lextab[i].regexp, ICAEOF) != 0)
      {
	 if (Lextab[i].ccode == Nope)
	 {
	    fprintf(fptr, "%s\t{\n", Lextab[i].regexp);
	    fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
	    fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
	    fprintf(fptr, "\t return(TOKEN%d);\n", i);
	    fprintf(fptr, "\t}\n\n");
	 }
	 else if (Lextab[i].ccode == Start)
	 {
	    fprintf(fptr, "%s\t{\n", Lextab[i].regexp);
	    fprintf(fptr, "\t BEGIN %s;\n", Lextab[i].context);
	    fprintf(fptr, "\t push(%s);\n", Lextab[i].context);
	    fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
	    fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
	    fprintf(fptr, "\t return(TOKEN%d);\n", i);
	    fprintf(fptr, "\t}\n\n");
	 }
      }
   }
   PrintCdataRules(fptr, 0);
   return;
}



/*
   FUNCTION: 		PrintTokens

   Synopsis: To print the list of token names in the generated 
   yacc code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintTokens(fptr)

FILE
   *fptr;

{
   int
      i;

   for (i = 0; i < Lexndx; i++)
   {
      /* do not generate code for ICAEOF */
      if (strcmp(Lextab[i].regexp, ICAEOF) != 0)
      {
	 fprintf(fptr, "TOKEN%d\n", i);
      }
   }
   return;
}



/*
   FUNCTION: 		PrintICAEOFTags

   Synopsis: To print the nonsegmenting (Implicit Segmenting) end
   tags implied by the EOF in the action routine of the document
   production.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


static void
PrintICAEOFTags(fptr)

FILE
   *fptr;

{
   int
      j;

   for ( j = 0; j < Yaccndx; j++)
   {
      if (strcmp(Lextab[Yacctab[j].lexndx].regexp, ICAEOF) == 0 &&
	  (Yacctab[j].root == False))
      {
	 fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[j].tag);
      }
   }
   return;
}



/*
   FUNCTION: 		PrintDocProds

   Synopsis: To print the yacc rules for document in the generated 
   yacc code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintDocProds(fptr)

FILE
   *fptr;

{
   int
      i, j;
   Boolean
      firstprod;

   firstprod = True;
   /* print document production if root tag has reg exp */
   for (i = 0; i < Yaccndx; i++)
   {
      if (Yacctab[i].root == True)
      {
	 if (firstprod)
	 {
	    fprintf(fptr, "document\t: ");
	    firstprod = False;
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
	 if (strcmp(Lextab[Yacctab[i].lexndx].regexp, ICAEOF) != 0)
	 {
	    if (Yacctab[i].segflag == True)
	    {
	       fprintf(fptr, "textortagsstar TOKEN%d textortagsplus ",
		       Yacctab[i].lexndx);
	       if (strcmp(Lextab[Yacctab[i].endndx].regexp, ICAEOF) != 0)
	       {
		  fprintf(fptr, "TOKEN%d textortagsstar", Yacctab[i].endndx);
	       }
	       fprintf(fptr, "\n\t\t{\n");
	       fprintf(fptr, "\t\t $$ = structconcat($1, ");
	       fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
	       fprintf(fptr, "$3, ");
	       PrintICAEOFTags(fptr);
	       fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	       if (strcmp(Lextab[Yacctab[i].endndx].regexp, ICAEOF) != 0)
	       {
		  fprintf(fptr, "$5, NULL);\n");
	       }
	       else
	       {
		  fprintf(fptr, "NULL);\n");
	       }
	    }
	    else
	    {
	       switch (Yacctab[i].tcode)
	       {
	       default:
		  break;

	       case Begin:
		  fprintf(fptr, "textortagsstar TOKEN%d textortagsplus\n",
			  Yacctab[i].lexndx);
		  fprintf(fptr, "\t\t{\n");
		  fprintf(fptr, "\t\t $$ = structconcat($1, ");
		  fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
		  fprintf(fptr, "$3, ");
		  PrintICAEOFTags(fptr);
		  fprintf(fptr, "NULL);\n");
		  break;

	       case EndT:
		  fprintf(fptr, "textortagsplus TOKEN%d textortagsstar\n", 
			  Yacctab[i].lexndx);
		  fprintf(fptr, "\t\t{\n");
		  fprintf(fptr, "\t\t $$ = structconcat($1, ");
		  PrintICAEOFTags(fptr);
		  fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
		  fprintf(fptr, "$3, NULL);\n");
		  break;
	       }
	    }
	 }
	 else
	 {
	    fprintf(fptr, "textortagsplus\n");
	    fprintf(fptr, "\t\t{\n");
	    fprintf(fptr, "\t\t $$ = structconcat($1, ");
	    PrintICAEOFTags(fptr);
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	 }
	 fprintf(fptr, "\t\t structprint($$);\n");
	 fprintf(fptr, "\t\t}\n");
      }
   }

   /*  The root tag had no reg exp, so print default production. */

   if (firstprod)
   {
      fprintf(fptr, "document\t: textortagsplus\n");
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t $$ = structconcat($1, ");
      PrintICAEOFTags(fptr);
      fprintf(fptr, "NULL);\n");
      fprintf(fptr, "\t\t structprint($$);\n");
      fprintf(fptr, "\t\t}\n");
   }
   fprintf(fptr, "\t\t;\n\n");
   return;
}



/*
   FUNCTION: 		PrintNSProds

   Synopsis: To print the yacc rules for Symbol, Processing 
   Insructions, Separator, and Implicit Segementing tags in the 
   generated yacc code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintNSProds(fptr)

FILE
   *fptr;

{
   int
      i;
   Boolean
      firstprod;

   firstprod = True;
   for (i = 0; i < Yaccndx; i++)
   {
      if ((Yacctab[i].segflag == False) && (Yacctab[i].root == False)
	  && (strcmp(Lextab[Yacctab[i].lexndx].regexp, ICAEOF) != 0))
      {
	 if (firstprod)
	 {
	    fprintf(fptr, "nonsegmenting\t: TOKEN%d\n", Yacctab[i].lexndx);
	    firstprod = False;
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| TOKEN%d\n", Yacctab[i].lexndx);
	 }
	 fprintf(fptr, "\t\t{\n");
	 switch (Yacctab[i].tcode)
	 {
	 default:
	    break;

	 case Begin:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"<%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case Comma:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case EndT:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case Symbol:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"&%s;\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case ProcIn:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"<?%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;
	 }
	 fprintf(fptr, "\t\t}\n");
      }
   }
   if (firstprod)
   {
      fprintf(fptr, "nonsegmenting\t: /* null */\n");
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t $$ = structconcat(structnew(\"\"));\n");
      fprintf(fptr, "\t\t}\n");
   }
   fprintf(fptr, "\t\t;\n\n");
   return;
}



/*
   FUNCTION: 		PrintSProds

   Synopsis: To print the yacc rules for Explicit Segementing 
   tags in the generated yacc code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintSProds(fptr)

FILE
   *fptr;

{
   int
      i;
   Boolean
      firstprod;

   firstprod = True;
   for (i = 0; i < Yaccndx; i++)
   {
      if ((Yacctab[i].segflag == True) && (Yacctab[i].root == False))
      {
	 if (firstprod)
	 {
	    fprintf(fptr, "segmenting\t: ");
	    firstprod = False;
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
	 if ((strcmp(NOREGEXP, Lextab[Yacctab[i].lexndx].regexp) != 0) &&
	     (strcmp(NOREGEXP, Lextab[Yacctab[i].endndx].regexp) != 0))
	 {
	    fprintf(fptr, "TOKEN%d textortagsstar ", Yacctab[i].lexndx);
	    if (strcmp(Lextab[Yacctab[i].endndx].regexp, ICAEOF) != 0)
	    {
	       fprintf(fptr, "TOKEN%d\n", Yacctab[i].endndx);
	    }
	    fprintf(fptr, "\t\t{\n");
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
	    fprintf(fptr, "$2, ");
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    fprintf(fptr, "\t\t}\n");
	 } /***** IF neither regular expression is empty */
      } /***** IF the segmenting flag is true and not the root */
   } /***** FOR each symbol in the Yacc table */

   if (firstprod)
   {
      fprintf(fptr, "segmenting\t: /* null */\n");
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t $$ = structconcat(structnew(\"\"));\n");
      fprintf(fptr, "\t\t}\n");
   } /***** IF nothing got generated for segmenting tags */

   fprintf(fptr, "\t\t;\n\n");
   return;
}



/*
   FUNCTION: 		PrintAmbCharProds

   Synopsis: To print the yacc rules for Ambiguous Characters
   in the generated yacc code.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintAmbCharProds(fptr)

FILE
   *fptr;

{
   int
      i;
   
   for (i = 0; i < Lexndx; i++)
   {
      if (Lextab[i].amb == True)
      {
	 fprintf(fptr, "\t\t| TOKEN%d\n", i);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat(structnew($1), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
      }
   }
   fprintf(fptr, "\t\t;\n\n");
   return;
}



/*
   FUNCTION: 		GenerateLex

   Synopsis: To generate the lex file from a retag spec file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
GenerateLex(filestr)

String
   filestr;

{
   FILE
      *fptr;
   String
      lexfile;
   
   lexfile = XtMalloc(strlen(filestr) + 8);
   if (lexfile == NULL)
   {
      fprintf(stderr, "GenerateLex: unable to allocate space\n");
      exit(2);
   }
   sprintf(lexfile, "%s.l", filestr);
   if  ((fptr = fopen(lexfile, "w")) == NULL)
   {
      XicaStatus(Statw, "Unable to open lex file [%s]\n", lexfile);
      XtFree(lexfile);
      return;
   }
   CreateCtxttab();
   InitChartab(Ctxtndx);
   Output(fptr, headlex);
   PrintStartConds(fptr);
   Output(fptr, midlex);
   PrintSCLexRules(fptr);
   PrintLexRules(fptr);
   Output(fptr, taillex);
   fclose(fptr);
   XtFree(lexfile);
   return;
}



/*
   FUNCTION: 		GenerateYacc

   Synopsis: To generate the yacc file from a retag spec file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
GenerateYacc(filestr)

String
   filestr;

{
   FILE
      *fptr;
   String
      yaccfile;
   
   yaccfile = XtMalloc(strlen(filestr) + 8);
   if (yaccfile == NULL)
   {
      fprintf(stderr, "GenerateYacc: unable to allocate space\n");
      exit(2);
   }
   sprintf(yaccfile, "%s.y", filestr);
   if  ((fptr = fopen(yaccfile, "w")) == NULL)
   {
      XicaStatus(Statw, "Unable to open yacc file [%s]\n", yaccfile);
      XtFree(yaccfile);
      return;
   }
   Output(fptr, headyacc);
   PrintTokens(fptr);
   Output(fptr, midyacc1);
   PrintDocProds(fptr);
   Output(fptr, midyacc2);
   PrintNSProds(fptr);
   PrintSProds(fptr);
   Output(fptr, midyacc3);
   PrintAmbCharProds(fptr);
   Output(fptr, tailyacc);
   fclose(fptr);
   XtFree(yaccfile);
   return;
}

